from models.model import Model
from models.db import db_conn
import models.ptms
import models.condensates


class ProteinModel(Model):
    def __init__(self):
        self.collection_name = "proteins"
        self.allowed_params = {
            'query': str,
            'species_tax_id': str,
            'ensembl_id': str,
            'ensembl_gene_id': str,
            'page': int,
            'size': int,
        }
        self.unique_key = "uniprot_id"
        self.attributes = [
            'name',
            'species_tax_id',
            'species_name',
            'sequence',
            'uniprot_id',
            'uniprot_readable_id',
            'gene_name',
            'ensembl_id',
            'ensembl_gene_id',
            'primary_data_source',
            'sequence',
            'antibody_link',
            'llps_ptms',
            'condensates',
            'function',
            'iupred_score'
        ]
        self.list_attributes = {
            'name': True,
            'gene_name': True,
            'species_tax_id': True,
            'species_name': True,
            'uniprot_id': True,
            'uniprot_readable_id': True
        }

    def fetch_record(self, unique_id):
        """
        Overriding base function for fetch record.
        Add custom and foreign key fields like "antibody_link" and "llps_ptms"

        :param(str) unique_id:
        :return(dict):
        """
        record = self.serialize_record(db_conn[self.collection_name].find_one({self.unique_key: unique_id}))
        if not record:
            return
        if record.get('llps_ptms'):
            record['llps_ptms'] = models.ptms.PtmModel().fetch_by_unique_keys(record.get('llps_ptms'))

        _, record['condensates'] = models.condensates.CondensateModel().fetch_all(proteins=record.get('uniprot_id'))
        record['antibody_link'] = "https://www.proteinatlas.org/{}/antibody".format(unique_id)
        return record
